@@ -1,6 +1,6 @@
 package Monitoring::GLPlugin;
 
-=head1 Monitoring::GLPlugin 
+=head1 Monitoring::GLPlugin
 
 Monitoring::GLPlugin - infrastructure functions to build a monitoring plugin
 
@@ -12,7 +12,7 @@ use File::Basename;
 use Digest::MD5 qw(md5_hex);
 use Errno;
 our $AUTOLOAD;
-our $VERSION = "1.1.1";
+*VERSION = \'1.4.1';
 
 use constant { OK => 0, WARNING => 1, CRITICAL => 2, UNKNOWN => 3 };
 
@@ -358,17 +358,17 @@ sub validate_args {
       $self->override_opt('statefilesdir', "/var/tmp/".$Monitoring::GLPlugin::plugin->{name});
     }
   }
-  $Monitoring::GLPlugin::plugin->{statefilesdir} = $self->opts->statefilesdir 
+  $Monitoring::GLPlugin::plugin->{statefilesdir} = $self->opts->statefilesdir
       if $self->opts->can("statefilesdir");
   if ($self->opts->can("warningx") && $self->opts->warningx) {
     foreach my $key (keys %{$self->opts->warningx}) {
-      $self->set_thresholds(metric => $key, 
+      $self->set_thresholds(metric => $key,
           warning => $self->opts->warningx->{$key});
     }
   }
   if ($self->opts->can("criticalx") && $self->opts->criticalx) {
     foreach my $key (keys %{$self->opts->criticalx}) {
-      $self->set_thresholds(metric => $key, 
+      $self->set_thresholds(metric => $key,
           critical => $self->opts->criticalx->{$key});
     }
   }
@@ -415,7 +415,7 @@ sub debug {
     printf "\n";
   }
   if ($self->{trace}) {
-    my $logfh = new IO::File;
+    my $logfh = IO::File->new();
     $logfh->autoflush(1);
     if ($logfh->open($tracefile, "a")) {
       $logfh->printf("%s: ", scalar localtime);
@@ -664,7 +664,9 @@ sub load_my_extension {
     }
     my $original_class = ref($self);
     my $original_init = $self->can("init");
+    $self->compatibility_class() if $self->can('compatibility_class');
     bless $self, "My$class";
+    $self->compatibility_methods() if $self->can('compatibility_methods');
     if ($self->isa("Monitoring::GLPlugin")) {
       my $new_init = $self->can("init");
       if ($new_init == $original_init) {
@@ -695,6 +697,45 @@ sub decode_password {
   return $password;
 }
 
+sub number_of_bits {
+  my $self = shift;
+  my $unit = shift;
+  # https://en.wikipedia.org/wiki/Data_rate_units
+  my $bits = {
+    'bit' => 1,			# Bit per second
+    'B' => 8,			# Byte per second, 8 bits per second
+    'kbit' => 1000,		# Kilobit per second, 1,000 bits per second
+    'kb' => 1000,		# Kilobit per second, 1,000 bits per second
+    'Kibit' => 1024,		# Kibibit per second, 1,024 bits per second
+    'kB' => 8000,		# Kilobyte per second, 8,000 bits per second
+    'KiB' => 8192,		# Kibibyte per second, 1,024 bytes per second
+    'Mbit' => 1000000,		# Megabit per second, 1,000,000 bits per second
+    'Mb' => 1000000,		# Megabit per second, 1,000,000 bits per second
+    'Mibit' => 1048576,		# Mebibit per second, 1,024 kibibits per second
+    'MB' => 8000000,		# Megabyte per second, 1,000 kilobytes per second
+    'MiB' => 8388608,		# Mebibyte per second, 1,024 kibibytes per second
+    'Gbit' => 1000000000,	# Gigabit per second, 1,000 megabits per second
+    'Gb' => 1000000000,		# Gigabit per second, 1,000 megabits per second
+    'Gibit' => 1073741824,	# Gibibit per second, 1,024 mebibits per second
+    'GB' => 8000000000,		# Gigabyte per second, 1,000 megabytes per second
+    'GiB' => 8589934592,	# Gibibyte per second, 8192 mebibits per second
+    'Tbit' => 1000000000000,	# Terabit per second, 1,000 gigabits per second
+    'Tb' => 1000000000000,	# Terabit per second, 1,000 gigabits per second
+    'Tibit' => 1099511627776,	# Tebibit per second, 1,024 gibibits per second
+    'TB' => 8000000000000,	# Terabyte per second, 1,000 gigabytes per second
+    # eigene kreationen
+    'Bits' => 1,
+    'KBi' => 1024,
+    'MBi' => 1024 * 1024,
+    'GBi' => 1024 * 1024 * 1024,
+  };
+  if (exists $bits->{$unit}) {
+    return $bits->{$unit};
+  } else {
+    return 0;
+  }
+}
+
 
 #########################################################
 # runtime methods
@@ -882,7 +923,7 @@ sub is_blacklisted {
         }
       }
     }
-  } 
+  }
   return $self->{blacklisted};
 }
 
@@ -965,18 +1006,18 @@ sub valdiff {
   }
   # lookback=99999, freeze=0(default)
   #  nimm den letzten lauf und schreib ihn nach {cold}
-  #  vergleich dann 
+  #  vergleich dann
   #    wenn es frozen gibt, vergleich frozen und den letzten lauf
   #    sonst den letzten lauf und den aktuellen lauf
   # lookback=99999, freeze=1
-  #  wird dann aufgerufen,wenn nach dem freeze=0 ein problem festgestellt wurde 
+  #  wird dann aufgerufen,wenn nach dem freeze=0 ein problem festgestellt wurde
   #     (also als 2.valdiff hinterher)
   #  schreib cold nach frozen
   # lookback=99999, freeze=2
   #  wird dann aufgerufen,wenn nach dem freeze=0 wieder alles ok ist
   #     (also als 2.valdiff hinterher)
   #  loescht frozen
-  #  
+  #
   my $last_values = $self->load_state(%params) || eval {
     my $empty_events = {};
     foreach (@keys) {
@@ -1026,7 +1067,7 @@ sub valdiff {
         } else {
           $last_values->{$_} = $last_values->{frozen}->{$_};
         }
-      } 
+      }
     } elsif ($mode eq "lookback") {
       # find a last_value in the history which fits lookback best
       # and overwrite $last_values->{$_} with historic data
@@ -1214,7 +1255,7 @@ sub save_state {
   if ((ref($params{save}) eq "HASH") && exists $params{save}->{timestamp}) {
     $params{save}->{localtime} = scalar localtime $params{save}->{timestamp};
   }
-  my $seekfh = new IO::File;
+  my $seekfh = IO::File->new();
   if ($seekfh->open($tmpfile, "w")) {
     $seekfh->printf("%s", Data::Dumper::Dumper($params{save}));
     $seekfh->flush();
@@ -1252,7 +1293,7 @@ sub load_state {
 #
 sub check_pidfile {
   my $self = shift;
-  my $fh = new IO::File;
+  my $fh = IO::File->new();
   if ($fh->open($self->{pidfile}, "r")) {
     my $pid = $fh->getline();
     $fh->close();
@@ -1295,7 +1336,7 @@ sub write_pidfile {
       } split(/\//, dirname($self->{pidfile}));
     }
   }
-  my $fh = new IO::File;
+  my $fh = IO::File->new();
   $fh->autoflush(1);
   if ($fh->open($self->{pidfile}, "w")) {
     $fh->printf("%s", $$);
@@ -1306,6 +1347,77 @@ sub write_pidfile {
   }
 }
 
+sub compatibility_methods {
+  my $self = shift;
+  # add_perfdata
+  # add_message
+  # nagios_exit
+  # ->{warningrange}
+  # ->{criticalrange}
+  # ...
+  $self->{warningrange} = ($self->get_thresholds())[0];
+  $self->{criticalrange} = ($self->get_thresholds())[1];
+  my $old_init = $self->can('init');
+  my %params = (
+    'mode' => join('::', split(/-/, $self->opts->mode)),
+    'name' => $self->opts->name,
+    'name2' => $self->opts->name2,
+  );
+  {
+    no strict 'refs';
+    no warnings 'redefine';
+    *{ref($self).'::init'} = sub {
+      $self->$old_init(%params);
+      $self->nagios(%params);
+    };
+    *{ref($self).'::add_nagios'} = \&{"Monitoring::GLPlugin::add_message"};
+    *{ref($self).'::add_nagios_ok'} = \&{"Monitoring::GLPlugin::add_ok"};
+    *{ref($self).'::add_nagios_warning'} = \&{"Monitoring::GLPlugin::add_warning"};
+    *{ref($self).'::add_nagios_critical'} = \&{"Monitoring::GLPlugin::add_critical"};
+    *{ref($self).'::add_nagios_unknown'} = \&{"Monitoring::GLPlugin::add_unknown"};
+    *{ref($self).'::add_perfdata'} = sub {
+      my $self = shift;
+      my $message = shift;
+      foreach my $perfdata (split(/\s+/, $message)) {
+      my ($label, $perfstr) = split(/=/, $perfdata);
+      my ($value, $warn, $crit, $min, $max) = split(/;/, $perfstr);
+      $value =~ /^([\d\.\-\+]+)(.*)$/;
+      $value = $1;
+      my $uom = $2;
+      $Monitoring::GLPlugin::plugin->add_perfdata(
+        label => $label,
+        value => $value,
+        uom => $uom,
+        warn => $warn,
+        crit => $crit,
+        min => $min,
+        max => $max,
+      );
+      }
+    };
+    *{ref($self).'::check_thresholds'} = sub {
+      my $self = shift;
+      my $value = shift;
+      my $defaultwarningrange = shift;
+      my $defaultcriticalrange = shift;
+      $Monitoring::GLPlugin::plugin->set_thresholds(
+          metric => 'default',
+          warning => $defaultwarningrange,
+          critical => $defaultcriticalrange,
+      );
+      $self->{warningrange} = ($self->get_thresholds())[0];
+      $self->{criticalrange} = ($self->get_thresholds())[1];
+      return $Monitoring::GLPlugin::plugin->check_thresholds(
+          metric => 'default',
+          value => $value,
+          warning => $defaultwarningrange,
+          critical => $defaultcriticalrange,
+      );
+    };
+  }
+}
+
+
 sub AUTOLOAD {
   my $self = shift;
   return if ($AUTOLOAD =~ /DESTROY/);
